#!/usr/bin/env python
# Copyright (c) 2012 The LibYuv Project Authors. All rights reserved.
#
# Use of this source code is governed by a BSD-style license
# that can be found in the LICENSE file in the root of the source
# tree. An additional intellectual property rights grant can be found
# in the file PATENTS.  All contributing project authors may
# be found in the AUTHORS file in the root of the source tree.

import os
import re
import sys

"""
Copied from Chrome's src/tools/valgrind/tsan/PRESUBMIT.py

See http://dev.chromium.org/developers/how-tos/depottools/presubmit-scripts
for more details on the presubmit API built into gcl.
"""

def CheckChange(input_api, output_api):
  """Checks the TSan suppressions files for bad suppressions."""

  # Add the path to the Chrome valgrind dir to the import path:
  tools_vg_path = os.path.join(input_api.PresubmitLocalPath(), '..', '..',
                               'valgrind')
  sys.path.append(tools_vg_path)
  import suppressions

  return suppressions.PresubmitCheck(input_api, output_api)

def CheckChangeOnUpload(input_api, output_api):
  return CheckChange(input_api, output_api)

def CheckChangeOnCommit(input_api, output_api):
  return CheckChange(input_api, output_api)

def GetPreferredTrySlaves():
  # We don't have any tsan slaves yet, so there's no use for this method.
  # When we have, the slave name(s) should be put into this list.
  return []



#!/usr/bin/env python
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
import sys
import subprocess
import os.path

def main():
  return subprocess.call([sys.executable, "../webrtc/trunk/build/dir_exists.py"] + sys.argv[1:])

if __name__ == '__main__':
  sys.exit(main())



#!/usr/bin/env python

# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

def writeSpecification(subject, issuer, extensions, fileBase):
    with open('%s.pem.certspec' % fileBase, 'w+') as f:
        f.write('issuer:%s\n' % issuer)
        f.write('subject:%s\n' % subject)
        if extensions:
            f.write('%s\n' % extensions)

def generateCommonEndEntityCertificates(issuer, issuerFileBase):
    writeSpecification('www.foo.com', issuer, None, 'cn-www.foo.com-%s' % issuerFileBase)
    writeSpecification('www.foo.org', issuer, None, 'cn-www.foo.org-%s' % issuerFileBase)
    writeSpecification('www.foo.com', issuer, 'extension:subjectAlternativeName:*.foo.org',
        'cn-www.foo.com-alt-foo.org-%s' % issuerFileBase)
    writeSpecification('www.foo.org', issuer, 'extension:subjectAlternativeName:*.foo.com',
        'cn-www.foo.org-alt-foo.com-%s' % issuerFileBase)
    writeSpecification('www.foo.com', issuer, 'extension:subjectAlternativeName:*.foo.com',
        'cn-www.foo.com-alt-foo.com-%s' % issuerFileBase)
    writeSpecification('www.foo.org', issuer, 'extension:subjectAlternativeName:*.foo.org',
        'cn-www.foo.org-alt-foo.org-%s' % issuerFileBase)
    writeSpecification('www.foo.com', issuer,
        'extension:subjectAlternativeName:*.foo.com,*.a.a.us,*.b.a.us',
        'cn-www.foo.com-alt-foo.com-a.a.us-b.a.us-%s' % issuerFileBase)
    writeSpecification('/C=US/O=bar/CN=www.foo.com', issuer, None,
        'cn-www.foo.com_o-bar_c-us-%s' % issuerFileBase)
    writeSpecification('/C=US/O=bar/CN=www.foo.org', issuer, None,
        'cn-www.foo.org_o-bar_c-us-%s' % issuerFileBase)
    writeSpecification('/C=US/O=bar/CN=www.foo.com', issuer,
        'extension:subjectAlternativeName:*.foo.org',
        'cn-www.foo.com_o-bar_c-us-alt-foo.org-%s' % issuerFileBase)
    writeSpecification('/C=US/O=bar/CN=www.foo.org', issuer,
        'extension:subjectAlternativeName:*.foo.com',
        'cn-www.foo.org_o-bar_c-us-alt-foo.com-%s' % issuerFileBase)
    writeSpecification('/C=US/O=bar/CN=www.foo.com', issuer,
        'extension:subjectAlternativeName:*.foo.com',
        'cn-www.foo.com_o-bar_c-us-alt-foo.com-%s' % issuerFileBase)
    writeSpecification('/C=US/O=bar/CN=www.foo.org', issuer,
        'extension:subjectAlternativeName:*.foo.org',
        'cn-www.foo.org_o-bar_c-us-alt-foo.org-%s' % issuerFileBase)
    writeSpecification('/C=US/O=bar/CN=www.foo.com', issuer,
        'extension:subjectAlternativeName:*.foo.com,*.a.a.us,*.b.a.us',
        'cn-www.foo.com_o-bar_c-us-alt-foo.com-a.a.us-b.a.us-%s' % issuerFileBase)

def generateCerts():
    # Unconstrainted root certificate to issue most intermediates
    caExtensions = 'extension:basicConstraints:cA,\nextension:keyUsage:cRLSign,keyCertSign'
    writeSpecification('ca-nc', 'ca-nc', caExtensions, 'ca-nc')

    # Intermediate with permitted subtree: dNSName:foo.com
    writeSpecification('int-nc-perm-foo.com-ca-nc', 'ca-nc',
        caExtensions + '\nextension:nameConstraints:permitted:foo.com',
        'int-nc-perm-foo.com-ca-nc')
    generateCommonEndEntityCertificates('int-nc-perm-foo.com-ca-nc', 'int-nc-perm-foo.com-ca-nc')

    # Intermediate with excluded subtree: dNSName:foo.com
    writeSpecification('int-nc-excl-foo.com-ca-nc', 'ca-nc',
        caExtensions + '\nextension:nameConstraints:excluded:foo.com',
        'int-nc-excl-foo.com-ca-nc')
    generateCommonEndEntityCertificates('int-nc-excl-foo.com-ca-nc', 'int-nc-excl-foo.com-ca-nc')

    # Intermediate with permitted subtree: directoryName:/C=US
    writeSpecification('int-nc-c-us-ca-nc', 'ca-nc',
        caExtensions + '\nextension:nameConstraints:permitted:/C=US', 'int-nc-c-us-ca-nc')
    generateCommonEndEntityCertificates('int-nc-c-us-ca-nc', 'int-nc-c-us-ca-nc')

    # Intermediate issued by previous intermediate with permitted subtree: dnsName:foo.com
    writeSpecification('/C=US/CN=int-nc-foo.com-int-nc-c-us-ca-nc', 'int-nc-c-us-ca-nc',
        caExtensions + '\nextension:nameConstraints:permitted:foo.com',
        'int-nc-foo.com-int-nc-c-us-ca-nc')
    generateCommonEndEntityCertificates('/C=US/CN=int-nc-foo.com-int-nc-c-us-ca-nc',
        'int-nc-foo.com-int-nc-c-us-ca-nc')

    # Intermediate with permitted subtree: dNSName:foo.com, directoryName:/C=US
    writeSpecification('int-nc-perm-foo.com_c-us-ca-nc', 'ca-nc',
        caExtensions + '\nextension:nameConstraints:permitted:foo.com,/C=US',
        'int-nc-perm-foo.com_c-us-ca-nc')
    generateCommonEndEntityCertificates('int-nc-perm-foo.com_c-us-ca-nc',
        'int-nc-perm-foo.com_c-us-ca-nc')

    # Intermediate with permitted subtree: directoryName:/C=UK
    writeSpecification('int-nc-perm-c-uk-ca-nc', 'ca-nc',
        caExtensions + '\nextension:nameConstraints:permitted:/C=UK', 'int-nc-perm-c-uk-ca-nc')
    generateCommonEndEntityCertificates('int-nc-perm-c-uk-ca-nc', 'int-nc-perm-c-uk-ca-nc')

    # Intermediate issued by previous intermediate in a different directoryName tree
    writeSpecification('/C=US/CN=int-c-us-int-nc-perm-c-uk-ca-nc', 'int-nc-perm-c-uk-ca-nc',
        caExtensions, 'int-c-us-int-nc-perm-c-uk-ca-nc')
    generateCommonEndEntityCertificates('/C=US/CN=int-c-us-int-nc-perm-c-uk-ca-nc',
        'int-c-us-int-nc-perm-c-uk-ca-nc')

    # Intermediate with permitted subtree: dNSName:foo.com, dNSName:a.us
    writeSpecification('int-nc-foo.com_a.us', 'ca-nc',
        caExtensions + '\nextension:nameConstraints:permitted:foo.com,a.us',
        'int-nc-foo.com_a.us')
    generateCommonEndEntityCertificates('int-nc-foo.com_a.us', 'int-nc-foo.com_a.us')

    # Intermediate issued by previous intermediate with permitted subtree: dNSName:foo.com
    writeSpecification('int-nc-foo.com-int-nc-foo.com_a.us', 'int-nc-foo.com_a.us',
        caExtensions + '\nextension:nameConstraints:permitted:foo.com',
        'int-nc-foo.com-int-nc-foo.com_a.us')
    generateCommonEndEntityCertificates('int-nc-foo.com-int-nc-foo.com_a.us',
        'int-nc-foo.com-int-nc-foo.com_a.us')

    # Root certificate with permitted subtree: dNSName:foo.com
    writeSpecification('ca-nc-perm-foo.com', 'ca-nc-perm-foo.com',
        caExtensions + '\nextension:nameConstraints:permitted:foo.com', 'ca-nc-perm-foo.com')

    # Intermediate without name constraints issued by constrained root
    writeSpecification('int-ca-nc-perm-foo.com', 'ca-nc-perm-foo.com', caExtensions,
        'int-ca-nc-perm-foo.com')
    generateCommonEndEntityCertificates('int-ca-nc-perm-foo.com', 'int-ca-nc-perm-foo.com')

generateCerts()



#!/usr/bin/env python
import site
import os
import logging
import argparse
import json
import sys
import hashlib
import requests
import tempfile
from boto.s3.connection import S3Connection
from mardor.marfile import MarFile

site.addsitedir("/home/worker/tools/lib/python")

from balrog.submitter.cli import NightlySubmitterV4, ReleaseSubmitterV4
from util.retry import retry, retriable

log = logging.getLogger(__name__)


def get_hash(content, hash_type="md5"):
    h = hashlib.new(hash_type)
    h.update(content)
    return h.hexdigest()


@retriable()
def download(url, dest, mode=None):
    log.debug("Downloading %s to %s", url, dest)
    r = requests.get(url)
    r.raise_for_status()

    bytes_downloaded = 0
    with open(dest, 'wb') as fd:
        for chunk in r.iter_content(4096):
            fd.write(chunk)
            bytes_downloaded += len(chunk)

    log.debug('Downloaded %s bytes', bytes_downloaded)
    if 'content-length' in r.headers:
        log.debug('Content-Length: %s bytes', r.headers['content-length'])
        if bytes_downloaded != int(r.headers['content-length']):
            raise IOError('Unexpected number of bytes downloaded')

    if mode:
        log.debug("chmod %o %s", mode, dest)
        os.chmod(dest, mode)


def verify_signature(mar, signature):
    log.info("Checking %s signature", mar)
    m = MarFile(mar, signature_versions=[(1, signature)])
    m.verify_signatures()


def verify_copy_to_s3(bucket_name, aws_access_key_id, aws_secret_access_key,
                      mar_url, mar_dest, signing_cert):
    conn = S3Connection(aws_access_key_id, aws_secret_access_key)
    bucket = conn.get_bucket(bucket_name)
    _, dest = tempfile.mkstemp()
    log.info("Downloading %s to %s...", mar_url, dest)
    download(mar_url, dest)
    log.info("Verifying the signature...")
    if not os.getenv("MOZ_DISABLE_MAR_CERT_VERIFICATION"):
        verify_signature(dest, signing_cert)
    for name in possible_names(mar_dest, 10):
        log.info("Checking if %s already exists", name)
        key = bucket.get_key(name)
        if not key:
            log.info("Uploading to %s...", name)
            key = bucket.new_key(name)
            # There is a chance for race condition here. To avoid it we check
            # the return value with replace=False. It should be not None.
            length = key.set_contents_from_filename(dest, replace=False)
            if length is None:
                log.warn("Name race condition using %s, trying again...", name)
                continue
            else:
                # key.make_public() may lead to race conditions, because
                # it doesn't pass version_id, so it may not set permissions
                bucket.set_canned_acl(acl_str='public-read', key_name=name,
                                      version_id=key.version_id)
                # Use explicit version_id to avoid using "latest" version
                return key.generate_url(expires_in=0, query_auth=False,
                                        version_id=key.version_id)
        else:
            if get_hash(key.get_contents_as_string()) == \
                    get_hash(open(dest).read()):
                log.info("%s has the same MD5 checksum, not uploading...",
                         name)
                return key.generate_url(expires_in=0, query_auth=False,
                                        version_id=key.version_id)
            log.info("%s already exists with different checksum, "
                     "trying another one...", name)

    raise RuntimeError("Cannot generate a unique name for %s", mar_dest)


def possible_names(initial_name, amount):
    """Generate names appending counter before extension"""
    prefix, ext = os.path.splitext(initial_name)
    return [initial_name] + ["{}-{}{}".format(prefix, n, ext) for n in
                             range(1, amount + 1)]


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument("--artifacts-url-prefix", required=True,
                        help="URL prefix for MAR")
    parser.add_argument("--manifest", required=True)
    parser.add_argument("-a", "--api-root", required=True,
                        help="Balrog API root")
    parser.add_argument("-d", "--dummy", action="store_true",
                        help="Add '-dummy' suffix to branch name")
    parser.add_argument("--signing-cert", required=True)
    parser.add_argument("-v", "--verbose", action="store_const",
                        dest="loglevel", const=logging.DEBUG,
                        default=logging.INFO)
    args = parser.parse_args()
    logging.basicConfig(format="%(asctime)s - %(levelname)s - %(message)s",
                        level=args.loglevel)
    logging.getLogger("requests").setLevel(logging.WARNING)
    logging.getLogger("boto").setLevel(logging.WARNING)

    balrog_username = os.environ.get("BALROG_USERNAME")
    balrog_password = os.environ.get("BALROG_PASSWORD")
    if not balrog_username and not balrog_password:
        raise RuntimeError("BALROG_USERNAME and BALROG_PASSWORD environment "
                           "variables should be set")

    s3_bucket = os.environ.get("S3_BUCKET")
    aws_access_key_id = os.environ.get("AWS_ACCESS_KEY_ID")
    aws_secret_access_key = os.environ.get("AWS_SECRET_ACCESS_KEY")
    if not (s3_bucket and aws_access_key_id and aws_secret_access_key):
        log.warn("Skipping S3 uploads...")
        uploads_enabled = False
    else:
        uploads_enabled = True

    manifest = json.load(open(args.manifest))
    auth = (balrog_username, balrog_password)

    for e in manifest:
        complete_info = [{
            "hash": e["to_hash"],
            "size": e["to_size"],
        }]
        partial_info = [{
            "hash": e["hash"],
            "size": e["size"],
        }]

        if "previousVersion" in e and "previousBuildNumber" in e:
            log.info("Release style balrog submission")
            partial_info[0]["previousVersion"] = e["previousVersion"]
            partial_info[0]["previousBuildNumber"] = e["previousBuildNumber"]
            submitter = ReleaseSubmitterV4(api_root=args.api_root, auth=auth,
                                           dummy=args.dummy)
            retry(lambda: submitter.run(
                platform=e["platform"], productName=e["appName"],
                version=e["toVersion"],
                build_number=e["toBuildNumber"],
                appVersion=e["version"], extVersion=e["version"],
                buildID=e["to_buildid"], locale=e["locale"],
                hashFunction='sha512',
                partialInfo=partial_info, completeInfo=complete_info,
            ))
        elif "from_buildid" in e and uploads_enabled:
            log.info("Nightly style balrog submission")
            partial_mar_url = "{}/{}".format(args.artifacts_url_prefix,
                                             e["mar"])
            complete_mar_url = e["to_mar"]
            dest_prefix = "{branch}/{buildid}".format(
                branch=e["branch"], buildid=e["to_buildid"])
            partial_mar_dest = "{}/{}".format(dest_prefix, e["mar"])
            complete_mar_filename = "{appName}-{branch}-{version}-" \
                                    "{platform}-{locale}.complete.mar"
            complete_mar_filename = complete_mar_filename.format(
                appName=e["appName"], branch=e["branch"],
                version=e["version"], platform=e["platform"],
                locale=e["locale"]
            )
            complete_mar_dest = "{}/{}".format(dest_prefix,
                                               complete_mar_filename)
            partial_info[0]["url"] = verify_copy_to_s3(
                s3_bucket, aws_access_key_id, aws_secret_access_key,
                partial_mar_url, partial_mar_dest, args.signing_cert)
            complete_info[0]["url"] = verify_copy_to_s3(
                s3_bucket, aws_access_key_id, aws_secret_access_key,
                complete_mar_url, complete_mar_dest, args.signing_cert)
            partial_info[0]["from_buildid"] = e["from_buildid"]
            submitter = NightlySubmitterV4(api_root=args.api_root, auth=auth,
                                           dummy=args.dummy)
            retry(lambda: submitter.run(
                platform=e["platform"], buildID=e["to_buildid"],
                productName=e["appName"], branch=e["branch"],
                appVersion=e["version"], locale=e["locale"],
                hashFunction='sha512', extVersion=e["version"],
                partialInfo=partial_info, completeInfo=complete_info),
                attempts=30, sleeptime=10, max_sleeptime=60,
            )
        else:
            raise RuntimeError("Cannot determine Balrog submission style")

if __name__ == '__main__':
    main()



#!/usr/bin/env python
#
# Copyright (c) 2012 The WebRTC project authors. All Rights Reserved.
#
# Use of this source code is governed by a BSD-style license
# that can be found in the LICENSE file in the root of the source
# tree. An additional intellectual property rights grant can be found
# in the file PATENTS.  All contributing project authors may
# be found in the AUTHORS file in the root of the source tree.

"""Runs an end-to-end audio quality test on Linux.

Expects the presence of PulseAudio virtual devices (null sinks). These are
configured as default devices for a VoiceEngine audio call. A PulseAudio
utility (pacat) is used to play to and record from the virtual devices.

The input reference file is then compared to the output file.
"""

import optparse
import os
import re
import shlex
import subprocess
import sys
import time

import perf.perf_utils

def main(argv):
  parser = optparse.OptionParser()
  usage = 'Usage: %prog [options]'
  parser.set_usage(usage)
  parser.add_option('--input', default='input.pcm', help='input PCM file')
  parser.add_option('--output', default='output.pcm', help='output PCM file')
  parser.add_option('--codec', default='ISAC', help='codec name')
  parser.add_option('--rate', default='16000', help='sample rate in Hz')
  parser.add_option('--channels', default='1', help='number of channels')
  parser.add_option('--play_sink', default='capture',
      help='name of PulseAudio sink to which to play audio')
  parser.add_option('--rec_sink', default='render',
      help='name of PulseAudio sink whose monitor will be recorded')
  parser.add_option('--harness',
      default=os.path.abspath(os.path.dirname(sys.argv[0]) +
          '/../../../out/Debug/audio_e2e_harness'),
      help='path to audio harness executable')
  parser.add_option('--compare',
                    help='command-line arguments for comparison tool')
  parser.add_option('--regexp',
                    help='regular expression to extract the comparison metric')
  options, _ = parser.parse_args(argv[1:])

  # Get the initial default capture device, to restore later.
  command = ['pacmd', 'list-sources']
  print ' '.join(command)
  proc = subprocess.Popen(command, stdout=subprocess.PIPE)
  output = proc.communicate()[0]
  if proc.returncode != 0:
    return proc.returncode
  default_source = re.search(r'(^  \* index: )([0-9]+$)', output,
                             re.MULTILINE).group(2)

  # Set the default capture device to be used by VoiceEngine. We unfortunately
  # need to do this rather than select the devices directly through the harness
  # because monitor sources don't appear in VoiceEngine except as defaults.
  #
  # We pass the render device for VoiceEngine to select because (for unknown
  # reasons) the virtual device is sometimes not used when the default.
  command = ['pacmd', 'set-default-source', options.play_sink + '.monitor']
  print ' '.join(command)
  retcode = subprocess.call(command, stdout=subprocess.PIPE)
  if retcode != 0:
    return retcode

  command = [options.harness, '--render=' + options.rec_sink,
      '--codec=' + options.codec, '--rate=' + options.rate]
  print ' '.join(command)
  voe_proc = subprocess.Popen(command)

  # If recording starts before there is data available, pacat sometimes
  # inexplicably adds a large delay to the start of the file. We wait here in
  # an attempt to prevent that, because VoE often takes some time to startup a
  # call.
  time.sleep(5)

  format_args = ['--format=s16le', '--rate=' + options.rate,
      '--channels=' + options.channels, '--raw']
  command = (['pacat', '-p', '-d', options.play_sink] + format_args +
      [options.input])
  print ' '.join(command)
  play_proc = subprocess.Popen(command)

  command = (['pacat', '-r', '-d', options.rec_sink + '.monitor'] +
      format_args + [options.output])
  print ' '.join(command)
  record_proc = subprocess.Popen(command)

  retcode = play_proc.wait()
  # If these ended early, an exception will be thrown here.
  record_proc.kill()
  voe_proc.kill()
  if retcode != 0:
    return retcode

  # Restore the initial default capture device.
  command = ['pacmd', 'set-default-source', default_source]
  print ' '.join(command)
  retcode = subprocess.call(command, stdout=subprocess.PIPE)
  if retcode != 0:
    return retcode

  if options.compare and options.regexp:
    command = shlex.split(options.compare) + [options.input, options.output]
    print ' '.join(command)
    proc = subprocess.Popen(command, stdout=subprocess.PIPE)
    output = proc.communicate()[0]
    if proc.returncode != 0:
      return proc.returncode

    # The list should only contain one item.
    value = ''.join(re.findall(options.regexp, output))

    perf.perf_utils.PrintPerfResult(graph_name='audio_e2e_score',
                                    series_name='e2e_score',
                                    data_point=value,
                                    units='MOS')  # Assuming we run PESQ.

  return 0

if __name__ == '__main__':
  sys.exit(main(sys.argv))



#!/usr/bin/env python
#
# Copyright (c) 2014 The WebRTC project authors. All Rights Reserved.
#
# Use of this source code is governed by a BSD-style license
# that can be found in the LICENSE file in the root of the source
# tree. An additional intellectual property rights grant can be found
# in the file PATENTS.  All contributing project authors may
# be found in the AUTHORS file in the root of the source tree.

# This file is (possibly, depending on python version) imported by
# gyp_webrtc when GYP_PARALLEL=1 and it creates sub-processes
# through the multiprocessing library.

# Importing in Python 2.6 (fixed in 2.7) on Windows doesn't search for
# imports that don't end in .py (and aren't directories with an
# __init__.py). This wrapper makes "import gyp_webrtc" work with
# those old versions and makes it possible to execute gyp_webrtc.py
# directly on Windows where the extension is useful.

import os

path = os.path.abspath(os.path.split(__file__)[0])
execfile(os.path.join(path, 'gyp_webrtc'))
